import errno
import network
import interfaces
import utime as time
from wssmqtt import *
from env import *

interfaces.init()

wifi = network.WLAN(network.STA_IF)
wifi.active(1)
wifi.connect(WIFI_SSID, WIFI_PASSWORD)


def do_connection():
    # Create an MQTT client
    keepalive = 5
    client = WSSMQTTClient(host=WSS_HOST, path=WSS_PATH,
                           keepalive=25, user=MQTT_USER, password=MQTT_PASSWORD)

    client.set_callback(interfaces.process_message)

    # Publish messages to and receive messages from the MQTT broker forever
    last_ping = time.ticks_ms()
    while True:
        if not client.is_connected():
            print('Connecting...')
            client.connect()
            last_ping = time.ticks_ms()

            for t in interfaces.list_topics():
                print(f'Subscribing to topic {t}...')
                client.subscribe(t)

            print('Subscribed')

        else:
            # Periodically ping the broker consistently with the "keepalive"
            # argument used when connecting.  If this isn't done, the broker will
            # disconnect when the client has been idle for 1.5x the keepalive
            # time.
            if (time.ticks_ms() - last_ping) >= keepalive * 1000:
                # print('Ping!')
                client.ping()
                last_ping = time.ticks_ms()

        # Every 5 minutes publish a message for fun.
        # client.publish(topic, 'Coming from MicroPython via a websocket @%d' % (time.ticks_ms(),))

        # Receive the next message if one is available, and handle the case that
        # the broker has disconnected.
        try:
            client.check_msg()
        except KeyboardInterrupt as e:
            client.disconnect()
            raise e
        except BaseException as e:
            if str(e) == '-1':
                client.disconnect()
                print('Connection closed')


def main():
    while True:
        try:
            print("Try connection...")
            do_connection()
        except KeyboardInterrupt as e:
            raise e
        except OSError as e:
            if e.errno == errno.ENOMEM:
                machine.reset()
            print(f"ERROR {e.__class__.__name__}")
            time.sleep(1)


main()
